import re
from bclib.context import Context
from ..predicate.predicate import Predicate


class Match (Predicate):
    """Create regex matching cheking predicate"""

    def __init__(self, expression: str, value: str) -> None:
        super().__init__(expression)
        self.__compiled_regex = re.compile(value)

    async def check_async(self, context: Context) -> bool:
        try:
            value = eval(self.exprossion, {}, {"context": context})
            return self.__compiled_regex.match(str(value)) != None
        except:  # pylint: disable=bare-except
            return False



from itertools import islice
import json
from typing import Any, TYPE_CHECKING, Coroutine, Iterator, Optional, Union
from aiohttp.web_response import ContentCoding

from ..context.request_context import RequestContext

if TYPE_CHECKING:
    from .. import dispatcher
    from .. import listener


class WebContext(RequestContext):
    def __init__(self, cms_object: dict,  dispatcher: 'dispatcher.IDispatcher',message_object: 'listener.WebMessage') -> None:
        super().__init__(cms_object, dispatcher)
        self.process_async = True
        self.__message = message_object
    
    async def start_stream_response_async(self,status: int = 200,
        reason: Optional[str] = 'OK',
        headers: Optional[dict] = None,)-> Coroutine[Any, Any, None]:
        await self.__message.start_stream_response_async(status,reason,headers)

    async def write_async(self,data:'bytes') -> Coroutine[Any, Any, None]:
        await  self.__message.write_async(data)

    async def drain_async(self) -> Coroutine[Any, Any, None]:
        await self.__message.drain_async()
    
    async def write_and_drain_async(self,data:'bytes') -> Coroutine[Any, Any, None]:
        await self.write_async(data)
        await self.drain_async()

    async def enable_compression(self,force: Optional[Union[bool, ContentCoding]] = None) ->  None:
        await self.__message.enable_compression(force)

    async def drain_array_async(self,data_list:Iterator,source_name:str, chunk_size:int,delimiter:str=',')-> Coroutine[Any, Any, None]:
        total_len =len(data_list)
        current:int = 0
        while current < total_len:
            temp_list = list(islice(data_list,current,current + chunk_size))
            current+= len(temp_list)
            data = {
                "sources": [
                {
                    "options": {
                    "tableName": source_name,
                    "mergeType": 1 #MergeType append,
                    },
                    "data": temp_list
                }],
            }
            await self.write_and_drain_async(f"{json.dumps(data)}{delimiter}".encode())



from typing import Dict, List, Optional

class LogObject:
    def __init__(self, schema_name: str, routing_key: Optional[str] = None, **kwargs) -> None:
        self.__items: Dict[str, List[List]] = dict()
        for key, value in kwargs.items():
            self.add_property(key, [value])
        self.schema_name = schema_name
        self.routingKey = routing_key

    def add_property(self, prp_title: str, answer: List):
        if len(answer) > 0:
            if prp_title not in self.__items:
                self.__items[prp_title] = list()
            self.__items[prp_title].append(answer)
    
    @property
    def properties(self):
        return self.__items


from typing import Any, List, Dict, Tuple

class LogSchema:
    def __init__(self, schema: Dict[str, Any]) -> None:
        self.schema_name = schema["schemaName"]
        self.schema_version = schema["schemaVersion"]
        self.lid = schema["lid"]
        self.schemaId = schema["schemaId"]
        self.paramUrl = schema["paramUrl"]
        self.properties: Dict[str, Tuple[int, bool, int, int, str]] = dict(
            [
                (
                    q["title"], (
                        q["prpId"], 
                        bool(q.get("multi", False)),
                        len(q["parts"]),
                        q["TypeID"] if "TypeID" in q else 0, 
                        q["source"] if "source" in q else None
                    )
                ) 
                for q in schema["questions"]
            ]
        )

    def get_answer(self, params: Dict[str, List[List]]):
        properties = list()
        for key, (id, multi, parts_count, typeid, source) in self.properties.items():
            try:
                if source is not None:
                    value = eval(source, {}, params)
                    properties.append({
                        "prpId": id,
                        "TypeID": typeid,
                        "answers": [
                            {
                                "parts": [
                                    {
                                        "part": 1,
                                        "values": [
                                            {
                                                "value": value
                                            }
                                        ]
                                    }
                                ]
                            }
                        ]
                    })
                else:
                    if key in params:
                        values = params[key]
                        if not multi:
                            values = values[:1]
                        prp_answers = list()
                        answer_parts = list()
                        part_index = 1
                        for parts_val in values:
                            parts_val = parts_val[:parts_count]
                            for part_val in parts_val:
                                answer_parts.append({
                                    "part": part_index,
                                    "values": [
                                        {
                                            "value": part_val
                                        }
                                    ]
                                })
                                part_index += 1
                            prp_answers.append({
                                "parts": answer_parts
                            })
                        properties.append({
                            "prpId": id,
                            "TypeID": typeid,
                            "answers": prp_answers
                        })
                        

            except:
                pass
        return {
            "schemaName": self.schema_name,
            "paramUrl": self.paramUrl,
            "schemaVersion": self.schema_version,
            "lid": self.lid,
            "schemaId": self.schemaId,
            "properties": properties
        }
